<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\Order;
use App\Models\Product;
use App\Models\User;
use App\Models\Category;
use App\Models\OrderItem;
use Carbon\Carbon;
use Illuminate\Support\Facades\DB;

class AnalysisController extends Controller
{
    /**
     * Display the analysis dashboard.
     */
    public function index()
    {
        return view('admin.analysis.index');
    }

    /**
     * Get sales analysis data.
     */
    public function salesAnalysis(Request $request)
    {
        $dateRange = $request->input('range', '30days');
        
        switch ($dateRange) {
            case '7days':
                $period = 7;
                break;
            case '30days':
                $period = 30;
                break;
            case '90days':
                $period = 90;
                break;
            case 'year':
                $period = 365;
                break;
            default:
                $period = 30;
        }

        $startDate = now()->subDays($period);
        
        $salesData = Order::select(
                DB::raw('DATE(created_at) as date'),
                DB::raw('SUM(total) as total_sales'),
                DB::raw('COUNT(*) as order_count')
            )
            ->where('created_at', '>=', $startDate)
            ->groupBy('date')
            ->orderBy('date', 'asc')
            ->get();

        $totalSales = Order::where('created_at', '>=', $startDate)->sum('total');
        $totalOrders = Order::where('created_at', '>=', $startDate)->count();
        $averageOrderValue = $totalOrders > 0 ? $totalSales / $totalOrders : 0;

        return response()->json([
            'sales_data' => $salesData,
            'total_sales' => $totalSales,
            'total_orders' => $totalOrders,
            'average_order_value' => $averageOrderValue,
            'period' => $period
        ]);
    }

    /**
     * Get product performance analysis.
     */
    public function productAnalysis()
    {
        $topProducts = Product::withCount('orderItems')
            ->withSum('orderItems', 'quantity')
            ->withSum('orderItems', 'price')
            ->orderByDesc('order_items_count')
            ->take(10)
            ->get();

        $topCategories = Category::withCount(['products' => function($query) {
                $query->has('orderItems');
            }])
            ->withSum(['products' => function($query){
                $query->has('orderItems');
            }], 'price')
            ->withSum(['products' => function($query){
                $query->has('orderItems');
            }], 'quantity')
            ->orderByDesc('products_count')
            ->take(5)
            ->get();

        $lowStockProducts = Product::where('stock', '<=', 10)
            ->orderBy('stock', 'asc')
            ->take(10)
            ->get();

        return response()->json([
            'top_products' => $topProducts,
            'top_categories' => $topCategories,
            'low_stock_products' => $lowStockProducts
        ]);
    }

    /**
     * Get customer analysis data.
     */
    public function customerAnalysis()
    {
        $newCustomers = User::where('created_at', '>=', now()->subDays(30))
            ->where('role', 'customer')
            ->count();

        $totalCustomers = User::where('role', 'customer')->count();

        $customerGrowth = User::where('role', 'customer')
            ->select(
                DB::raw('DATE(created_at) as date'),
                DB::raw('COUNT(*) as count')
            )
            ->where('created_at', '>=', now()->subDays(30))
            ->groupBy('date')
            ->orderBy('date', 'asc')
            ->get();

        $topCustomers = User::where('role', 'customer')
            ->withSum(['orders' => function($query) {
                $query->where('status', 'completed');
            }], 'total')
            ->orderByDesc('orders_sum_total')
            ->take(10)
            ->get();

        return response()->json([
            'total_customers' => $totalCustomers,
            'new_customers' => $newCustomers,
            'customer_growth' => $customerGrowth,
            'top_customers' => $topCustomers
        ]);
    }

    /**
     * Get inventory analysis data.
     */
    public function inventoryAnalysis()
    {
        $totalProducts = Product::count();
        $outOfStockProducts = Product::where('stock', 0)->count();
        $lowStockProducts = Product::whereBetween('stock', [1, 10])->count();
        
        $outOfStockPercentage = $totalProducts > 0 ? ($outOfStockProducts / $totalProducts) * 100 : 0;
        $lowStockPercentage = $totalProducts > 0 ? ($lowStockProducts / $totalProducts) * 100 : 0;

        // Average stock per category
        $avgStockByCategory = Product::select('category_id', DB::raw('AVG(stock) as avg_stock'))
            ->groupBy('category_id')
            ->get();

        return response()->json([
            'total_products' => $totalProducts,
            'out_of_stock_products' => $outOfStockProducts,
            'low_stock_products' => $lowStockProducts,
            'out_of_stock_percentage' => round($outOfStockPercentage, 2),
            'low_stock_percentage' => round($lowStockPercentage, 2),
            'average_stock_by_category' => $avgStockByCategory
        ]);
    }

    /**
     * Export analysis data.
     */
    public function exportData(Request $request)
    {
        $type = $request->input('type');
        $format = $request->input('format', 'csv');
        $dateRange = $request->input('range', '30days');

        // Set date range
        switch ($dateRange) {
            case '7days':
                $startDate = now()->subDays(7);
                break;
            case '30days':
                $startDate = now()->subDays(30);
                break;
            case '90days':
                $startDate = now()->subDays(90);
                break;
            case 'year':
                $startDate = now()->subYear();
                break;
            default:
                $startDate = now()->subDays(30);
        }

        // Get data based on type
        switch ($type) {
            case 'sales':
                $data = Order::where('created_at', '>=', $startDate)
                    ->select('order_number', 'total', 'status', 'created_at')
                    ->get();
                break;

            case 'products':
                $data = Product::withCount('orderItems')
                    ->withSum('orderItems', 'quantity')
                    ->withSum('orderItems', 'price')
                    ->get();
                break;

            case 'customers':
                $data = User::where('is_admin', false)
                    ->withCount(['orders' => function($query) {
                        $query->where('status', 'completed');
                    }])
                    ->get();
                break;

            default:
                return response()->json(['error' => 'Invalid export type'], 400);
        }

        // Export based on format
        if ($format === 'csv') {
            return $this->exportToCSV($data, $type);
        } else if ($format === 'excel') {
            return $this->exportToExcel($data, $type);
        } else {
            return response()->json(['error' => 'Unsupported format'], 400);
        }
    }

    /**
     * Export data to CSV format.
     */
    private function exportToCSV($data, $type)
    {
        $timestamp = now()->format('Ymd_His');
        $filename = "analysis_{$type}_export_{$timestamp}.csv";
        
        $headers = array(
            "Content-type" => "text/csv",
            "Content-Disposition" => "attachment; filename={$filename}",
            "Pragma" => "no-cache",
            "Cache-Control" => "must-revalidate, post-check=0, pre-check=0",
            "Expires" => "0"
        );

        $callback = function() use ($data, $type) {
            $file = fopen('php://output', 'w');
            
            // Add headers based on export type
            switch ($type) {
                case 'sales':
                    fputcsv($file, ['Order Number', 'Total Amount', 'Status', 'Date']);
                    break;
                case 'products':
                    fputcsv($file, ['Product Name', 'Category', 'Total Sold Quantity', 'Total Sales', 'Stock']);
                    break;
                case 'customers':
                    fputcsv($file, ['Name', 'Email', 'Total Orders', 'Total Spent', 'Last Order Date']);
                    break;
            }
            
            // Add data rows
            foreach ($data as $item) {
                switch ($type) {
                    case 'sales':
                        fputcsv($file, [
                            $item->order_number,
                            $item->total,
                            $item->status,
                            $item->created_at->format('Y-m-d H:i:s')
                        ]);
                        break;
                    case 'products':
                        fputcsv($file, [
                            $item->name,
                            optional($item->category)->name ?? 'N/A',
                            $item->order_items_sum_quantity ?: 0,
                            $item->order_items_sum_price ?: 0,
                            $item->stock
                        ]);
                        break;
                    case 'customers':
                        $lastOrder = optional($item->orders->sortByDesc('created_at')->first())->created_at;
                        
                        fputcsv($file, [
                            $item->name,
                            $item->email,
                            $item->orders_count ?: 0,
                            $item->orders_sum_total ?: 0,
                            $lastOrder ? $lastOrder->format('Y-m-d H:i:s') : 'N/A'
                        ]);
                        break;
                }
            }
            
            fclose($file);
        };
        
        return response()->streamDownload($callback, $filename, $headers);
    }

    /**
     * Export data to Excel format.
     */
    private function exportToExcel($data, $type)
    {
        // This would use a package like Maatwebsite Laravel-Excel to export to Excel format
        // As this requires package installation, we're returning a notification
        return response()->json([
            'message' => 'Excel export requires the Laravel-Excel package to be installed',
            'note' => 'Please install the package via Composer to enable Excel export'
        ]);
    }
}