<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Models\Message;
use App\Models\User;
use App\Models\Order;
use App\Events\NewMessage;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Validator;
use App\Models\Chat;

class ChatController extends Controller
{
    public function index()
    {
        return view('admin.chat.index');
    }

    public function getConversations()
    {
        $conversations = Message::where('sender_id', Auth::id())
            ->orWhere('receiver_id', Auth::id())
            ->with(['sender', 'receiver'])
            ->get()
            ->groupBy(function ($message) {
                return $message->sender_id === Auth::id() 
                    ? $message->receiver_id 
                    : $message->sender_id;
            })
            ->map(function ($messages) {
                $otherUser = $messages->first()->sender_id === Auth::id()
                    ? $messages->first()->receiver
                    : $messages->first()->sender;
                
                return [
                    'id' => $otherUser->id,
                    'name' => $otherUser->name,
                    'avatar' => $otherUser->avatar,
                    'last_message' => $messages->sortByDesc('created_at')->first()->content,
                    'last_message_time' => $messages->sortByDesc('created_at')->first()->created_at,
                    'unread' => $messages->where('receiver_id', Auth::id())
                        ->whereNull('seen_at')
                        ->count()
                ];
            })
            ->values();

        return response()->json($conversations);
    }

    public function getMessages(User $user)
    {
        $messages = Message::where(function ($query) use ($user) {
            $query->where('sender_id', Auth::id())
                ->where('receiver_id', $user->id);
        })->orWhere(function ($query) use ($user) {
            $query->where('sender_id', $user->id)
                ->where('receiver_id', Auth::id());
        })
        ->with(['sender', 'receiver'])
        ->orderBy('created_at', 'asc')
        ->get();

        // Mark messages as seen
        Message::where('sender_id', $user->id)
            ->where('receiver_id', Auth::id())
            ->whereNull('seen_at')
            ->update(['seen_at' => now()]);

        return response()->json($messages);
    }

    public function sendMessage(Request $request)
    {
        $request->validate([
            'receiver_id' => 'required|exists:users,id',
            'message' => 'required|string',
            'type' => 'required|in:text,image,file,voice',
            'file' => 'nullable|file|max:10240'
        ]);

        $message = Message::create([
            'sender_id' => Auth::id(),
            'receiver_id' => $request->receiver_id,
            'content' => $request->message,
            'type' => $request->type,
            'file_path' => $request->hasFile('file') ? $request->file('file')->store('chat-files') : null
        ]);

        return response()->json($message->load(['sender', 'receiver']));
    }

    public function search(Request $request)
    {
        $query = $request->input('q');
        
        $users = User::where('id', '!=', Auth::id())
            ->where(function ($q) use ($query) {
                $q->where('name', 'like', "%{$query}%")
                    ->orWhere('email', 'like', "%{$query}%");
            })
            ->limit(10)
            ->get();
        
        return response()->json($users);
    }

    public function markAsSeen(Message $message)
    {
        $message->markAsSeen();
        return response()->json(['success' => true]);
    }

    public function getUnreadCount()
    {
        $count = Message::where('receiver_id', auth()->id())
            ->whereNull('seen_at')
            ->count();

        return response()->json(['count' => $count]);
    }

    public function thread($userId)
    {
        $authId = Auth::id();
        $messages = Message::where(function ($q) use ($authId, $userId) {
            $q->where('sender_id', $authId)->where('receiver_id', $userId);
        })->orWhere(function ($q) use ($authId, $userId) {
            $q->where('sender_id', $userId)->where('receiver_id', $authId);
        })
        ->orderBy('created_at', 'asc')
        ->get();

        // Mark messages as seen
        Message::where('sender_id', $userId)
            ->where('receiver_id', $authId)
            ->whereNull('seen_at')
            ->update(['seen_at' => now()]);

        return response()->json($messages);
    }

    public function send(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'receiver_id' => 'required|exists:users,id',
            'message' => 'required|string',
            'type' => 'required|in:text,image,video,file,voice',
            'file' => 'nullable|file|max:10240' // 10MB max
        ]);

        if ($validator->fails()) {
            return response()->json(['errors' => $validator->errors()], 422);
        }

        $filePath = null;
        if ($request->hasFile('file')) {
            $file = $request->file('file');
            $type = $request->type;
            $folder = 'chat/' . $type . 's';
            $filePath = $file->store($folder, 'public');
        }

        // If it's a file message and no message text is provided, set a default message
        if ($request->type !== 'text' && empty($request->message)) {
            $request->merge(['message' => 'Sent a ' . $request->type]);
        }

        $message = Message::create([
            'sender_id' => Auth::id(),
            'receiver_id' => $request->receiver_id,
            'message' => $request->message,
            'type' => $request->type,
            'file_path' => $filePath
        ]);

        return response()->json($message);
    }

    public function conversations()
    {
        $authId = Auth::id();
        $conversations = Message::selectRaw('IF(sender_id = ?, receiver_id, sender_id) as user_id', [$authId])
            ->where(function($q) use ($authId) {
                $q->where('sender_id', $authId)->orWhere('receiver_id', $authId);
            })
            ->groupBy('user_id')
            ->get()
            ->map(function($row) use ($authId) {
                $user = User::find($row->user_id);
                $lastMessage = Message::where(function($q) use ($authId, $row) {
                        $q->where('sender_id', $authId)->where('receiver_id', $row->user_id);
                    })->orWhere(function($q) use ($authId, $row) {
                        $q->where('sender_id', $row->user_id)->where('receiver_id', $authId);
                    })
                    ->orderByDesc('created_at')
                    ->first();
                $unread = Message::where('sender_id', $row->user_id)
                    ->where('receiver_id', $authId)
                    ->whereNull('seen_at')
                    ->count();
                return [
                    'id' => $user->id,
                    'name' => $user->name,
                    'email' => $user->email,
                    'last_message' => $lastMessage ? ($lastMessage->type === 'text' ? $lastMessage->message : ucfirst($lastMessage->type)) : '',
                    'last_message_time' => $lastMessage ? $lastMessage->created_at : null,
                    'unread' => $unread,
                    'online' => false
                ];
            });

        return response()->json($conversations);
    }

    public function deleteMessage($messageId)
    {
        $message = Message::findOrFail($messageId);
        
        if ($message->sender_id !== Auth::id()) {
            return response()->json(['error' => 'Unauthorized'], 403);
        }

        if ($message->file_path) {
            Storage::disk('public')->delete($message->file_path);
        }

        $message->delete();
        return response()->json(['success' => true]);
    }

    public function toggleReaction($messageId, Request $request)
    {
        $message = Message::findOrFail($messageId);
        $emoji = $request->input('emoji');

        // Implement reaction logic here
        // This is a placeholder - you'll need to implement the actual reaction storage
        return response()->json([
            'reactions' => [],
            'userReactions' => []
        ]);
    }
} 