<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Models\Order;
use App\Models\Product;
use App\Models\User;
use App\Models\Category;
use App\Models\OrderItem;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Schema;
use Illuminate\Support\Facades\DB;
use Carbon\Carbon;

class DashboardController extends Controller
{
    public function index()
    {
        // Basic Stats
        $totalProducts = Product::count();
        $totalCustomers = User::where('is_admin', false)->count();
        $totalOrders = Order::count();
        $totalRevenueRow = \DB::table('total_revenue')->orderByDesc('created_at')->first();
        $totalRevenue = $totalRevenueRow ? $totalRevenueRow->total_amount : 0;

        // Recent Orders with Customer Info
        $recentOrders = Order::with(['user', 'items.variation.product'])
            ->latest()
            ->take(10)
            ->get();

        // Sales Analysis (Last 30 Days)
        $thirtyDaysAgo = Carbon::now()->subDays(30);
        $salesData = Order::where('created_at', '>=', $thirtyDaysAgo)
            ->where('status', 'completed')
            ->select(
                DB::raw('DATE(created_at) as date'),
                DB::raw('SUM(total) as total_sales'),
                DB::raw('COUNT(*) as order_count')
            )
            ->groupBy('date')
            ->orderBy('date')
            ->get();

        // Top Selling Products
        $topProducts = Product::with(['category', 'variations'])
            ->withCount(['orderItems' => function($query) {
                $query->whereHas('order', function($q) {
                    $q->where('status', 'completed');
                });
            }])
            ->withSum(['orderItems' => function($query) {
                $query->whereHas('order', function($q) {
                    $q->where('status', 'completed');
                });
            }], 'quantity')
            ->withSum(['orderItems' => function($query) {
                $query->whereHas('order', function($q) {
                    $q->where('status', 'completed');
                });
            }], DB::raw('quantity * price'))
            ->orderByDesc('order_items_sum_quantity')
            ->take(5)
            ->get();

        // Top Customers (join with customer_spent for total_spent)
        $topCustomers = User::where('is_admin', false)
            ->leftJoin('customer_spent', 'users.id', '=', 'customer_spent.user_id')
            ->with(['orders' => function($query) {
                $query->where('status', 'completed')
                    ->latest();
            }])
            ->withCount(['orders' => function($query) {
                $query->where('status', 'completed');
            }])
            ->select('users.*', 'customer_spent.total_spent')
            ->orderByDesc('customer_spent.total_spent')
            ->take(10)
            ->get();

        // Low Stock Products
        $lowStockProducts = Product::with(['category', 'variations' => function($query) {
                $query->where('stock', '<=', 10);
            }])
            ->whereHas('variations', function($query) {
                $query->where('stock', '<=', 10);
            })
            ->take(10)
            ->get();

        // New Customers (Last 30 Days)
        $newCustomers = User::where('is_admin', false)
            ->where('created_at', '>=', $thirtyDaysAgo)
            ->count();

        // Out of Stock Products
        $outOfStockProducts = Product::whereHas('variations', function($query) {
                $query->where('stock', 0);
            })
            ->count();

        return view('admin.dashboard', compact(
            'totalProducts',
            'totalOrders',
            'totalCustomers',
            'totalRevenue',
            'recentOrders',
            'salesData',
            'topProducts',
            'topCustomers',
            'lowStockProducts',
            'newCustomers',
            'outOfStockProducts'
        ));
    }

    /**
     * Get total sales from total_sales table
     */
    public function totalSales()
    {
        $totalSales = \DB::table('total_sales')->orderByDesc('created_at')->first();
        return response()->json(['total_sales' => $totalSales ? $totalSales->total_amount : 0]);
    }

    /**
     * Get total revenue from total_revenue table
     */
    public function totalRevenue()
    {
        $totalRevenue = \DB::table('total_revenue')->orderByDesc('created_at')->first();
        return response()->json(['total_revenue' => $totalRevenue ? $totalRevenue->total_amount : 0]);
    }

    /**
     * Get total spent by each customer from customer_spent table
     */
    public function totalSpent()
    {
        $customerSpent = \DB::table('customer_spent')->get(['user_id', 'total_spent']);
        return response()->json(['customer_spent' => $customerSpent]);
    }
} 