<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Models\Product;
use App\Models\Category;
use App\Models\VariationType;
use App\Models\VariationOptionLookup;
use Illuminate\Http\Request;
use Illuminate\Support\Str;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\DB;
use App\Models\Brand;

class ProductController extends Controller
{
    public function index()
    {
        $products = Product::with('category')->paginate(10);
        return view('admin.products.index', compact('products'));
    }

    public function create()
    {
        $categories = Category::root()->ordered()->get();
        $variationTypes = VariationType::with(['options' => function($query) {
            $query->where('is_active', true)->orderBy('order');
        }])->where('is_active', true)->get();
        
        return view('admin.products.create', compact('categories', 'variationTypes'));
    }

    public function store(Request $request)
    {
        \Log::info('Product creation request received', [
            'request_data' => $request->all(),
            'files' => $request->hasFile('images') ? 'Has files' : 'No files'
        ]);

        try {
            $validated = $request->validate([
                'name_en' => 'required|string|max:255',
                'name_ar' => 'required|string|max:255',
                'description_en' => 'required|string',
                'description_ar' => 'required|string',
                'category_id' => 'required|exists:categories,id',
                'is_active' => 'boolean',
                'images.*' => 'image|mimes:jpeg,png,jpg,gif|max:2048',
                'variation_types' => 'required|array',
                'variation_types.*' => 'exists:variation_types,id'
            ]);

            \Log::info('Validation passed', ['validated_data' => $validated]);

            DB::beginTransaction();
            try {
                $product = Product::create([
                    'name_en' => $validated['name_en'],
                    'name_ar' => $validated['name_ar'],
                    'description_en' => $validated['description_en'],
                    'description_ar' => $validated['description_ar'],
                    'category_id' => $validated['category_id'],
                    'is_active' => $validated['is_active'] ?? true,
                    'slug' => Str::slug($validated['name_en'])
                ]);

                \Log::info('Product created', ['product_id' => $product->id]);

                // Attach variation types
                $product->variationTypes()->attach($validated['variation_types']);
                \Log::info('Variation types attached');

                // Handle images
                if ($request->hasFile('images')) {
                    foreach ($request->file('images') as $image) {
                        $path = $image->store('products', 'public');
                        $product->images()->create(['image_path' => $path]);
                        \Log::info('Image stored', ['path' => $path]);
                    }
                }

                DB::commit();
                \Log::info('Transaction committed successfully');
                
                return redirect()->route('admin.products.index')
                    ->with('success', 'Product created successfully. Please add variations.');
            } catch (\Exception $e) {
                DB::rollBack();
                \Log::error('Error during product creation', [
                    'error' => $e->getMessage(),
                    'trace' => $e->getTraceAsString()
                ]);
                return back()->with('error', 'Failed to create product: ' . $e->getMessage());
            }
        } catch (\Illuminate\Validation\ValidationException $e) {
            \Log::error('Validation failed', [
                'errors' => $e->errors(),
                'input' => $request->all()
            ]);
            return back()->withErrors($e->errors())->withInput();
        } catch (\Exception $e) {
            \Log::error('Unexpected error during product creation', [
                'error' => $e->getMessage(),
                'trace' => $e->getTraceAsString()
            ]);
            return back()->with('error', 'An unexpected error occurred: ' . $e->getMessage());
        }
    }

    public function show(Product $product)
    {
        return view('admin.products.show', compact('product'));
    }

    public function edit(Product $product)
    {
        $categories = Category::root()->ordered()->get();
        return view('admin.products.edit', compact('product', 'categories'));
    }

    public function update(Request $request, Product $product)
    {
        $request->validate([
            'category_id' => 'exists:categories,id',
            'name_en' => 'string|max:255',
            'name_ar' => 'string|max:255',
            'description_en' => 'string',
            'description_ar' => 'string',
            'is_featured' => 'boolean',
            'is_active' => 'boolean',
            'stock_threshold' => 'integer|min:0'
        ]);

        $product->update($request->all());

        if ($request->has('name_en')) {
            $product->slug = Str::slug($request->name_en);
            $product->save();
        }

        return redirect()->route('admin.products.index')
            ->with('success', 'Product updated successfully');
    }

    public function destroy(Product $product)
    {
        // Delete associated images from storage
        foreach ($product->images as $image) {
            Storage::disk('public')->delete($image->image_path);
        }

        $product->delete();

        return response()->json([
            'message' => 'Product deleted successfully'
        ]);
    }
} 