<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Models\Product;
use App\Models\Variation;
use App\Models\VariationType;
use App\Models\VariationOptionLookup;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Storage;

class VariationController extends Controller
{
    public function index(Request $request, Product $product = null)
    {
        if ($product) {
            // Show variations for a specific product
            $variations = Variation::with(['product', 'options'])
                ->where('product_id', $product->id)
                ->paginate(10);
            $types = VariationType::all();
            $options = VariationOptionLookup::with('type')->orderBy('variation_type_id')->orderBy('order')->get();
            
            return view('admin.variations.index', compact('variations', 'product', 'types', 'options'));
        } else {
            // Show all variations
            $variations = Variation::with(['product', 'options'])->paginate(10);
            $products = Product::select('id', 'name_en')->orderBy('name_en')->get();
            $types = VariationType::all();
            $options = VariationOptionLookup::with('type')->orderBy('variation_type_id')->orderBy('order')->get();
            
            return view('admin.variations.index', compact('variations', 'products', 'types', 'options'));
        }
    }

    public function create(Request $request, Product $product = null)
    {
        $types = VariationType::all();
        $options = VariationOptionLookup::with('type')->orderBy('variation_type_id')->orderBy('order')->get();
        
        if ($product) {
            // Create variation for a specific product
            return view('admin.variations.create', compact('product', 'types', 'options'));
        } else {
            // Create variation with product selection
            $products = Product::select('id', 'name_en')->orderBy('name_en')->get();
            return view('admin.variations.create', compact('products', 'types', 'options'));
        }
    }

    public function store(Request $request, Product $product = null)
    {
        $validated = $request->validate([
            'product_id' => $product ? 'required|exists:products,id|in:' . $product->id : 'required|exists:products,id',
            'sku' => 'required|string|max:255|unique:variations',
            'barcode' => 'nullable|string|max:255|unique:variations',
            'price' => 'required|numeric|min:0',
            'stock' => 'required|integer|min:0',
            'options' => 'required|array',
            'options.*.type_id' => 'required|exists:variation_types,id',
            'options.*.option_id' => 'required|exists:variation_option_lookups,id',
            'image' => 'nullable|image|mimes:jpeg,png,jpg,gif|max:2048',
            'is_active' => 'boolean',
        ]);

        $variation = Variation::create($validated);

        if ($request->hasFile('image')) {
            $path = $request->file('image')->store('variations', 'public');
            $variation->image = $path;
            $variation->save();
        }

        // Extract option IDs from the options array
        $optionIds = collect($request->options)->pluck('option_id')->toArray();
        $variation->options()->attach($optionIds);

        if ($product) {
            return redirect()->route('admin.products.variations.index', $product)
                ->with('success', 'Variation created successfully.');
        } else {
            return redirect()->route('admin.variations.index')
                ->with('success', 'Variation created successfully.');
        }
    }

    public function edit(Product $product, Variation $variation)
    {
        // Ensure the variation belongs to the specified product
        if ($variation->product_id !== $product->id) {
            abort(404);
        }

        $variation->load('product');
        $variationTypes = VariationType::with('options')->get();
        return view('admin.variations.edit', compact('variation', 'variationTypes'));
    }

    public function update(Request $request, Product $product, Variation $variation)
    {
        // Ensure the variation belongs to the specified product
        if ($variation->product_id !== $product->id) {
            abort(404);
        }

        $validated = $request->validate([
            'sku' => 'required|string|max:255|unique:variations,sku,' . $variation->id,
            'barcode' => 'nullable|string|max:255|unique:variations,barcode,' . $variation->id,
            'price' => 'required|numeric|min:0',
            'stock' => 'required|integer|min:0',
            'options' => 'required|array',
            'options.*.type_id' => 'required|exists:variation_types,id',
            'options.*.option_id' => 'required|exists:variation_option_lookups,id',
            'image' => 'nullable|image|mimes:jpeg,png,jpg,gif|max:2048',
            'is_active' => 'boolean',
        ]);

        $variation->update($validated);

        if ($request->hasFile('image')) {
            if ($variation->image) {
                Storage::disk('public')->delete($variation->image);
            }
            $path = $request->file('image')->store('variations', 'public');
            $variation->image = $path;
            $variation->save();
        }

        // Extract option IDs from the options array
        $optionIds = collect($request->options)->pluck('option_id')->toArray();
        $variation->options()->sync($optionIds);

        return redirect()->route('admin.products.variations.index', $product)
            ->with('success', 'Variation updated successfully.');
    }

    public function destroy(Product $product, Variation $variation)
    {
        // Ensure the variation belongs to the specified product
        if ($variation->product_id !== $product->id) {
            abort(404);
        }

        if ($variation->image) {
            Storage::disk('public')->delete($variation->image);
        }
        
        $variation->delete();

        return redirect()->route('admin.products.variations.index', $product)
            ->with('success', 'Variation deleted successfully.');
    }
} 