<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Models\Message;
use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\Validator;

class ChatController extends Controller
{
    // Fetch conversation with a user
    public function thread($userId)
    {
        $authId = Auth::id();
        $messages = Message::where(function ($q) use ($authId, $userId) {
            $q->where('sender_id', $authId)->where('receiver_id', $userId);
        })->orWhere(function ($q) use ($authId, $userId) {
            $q->where('sender_id', $userId)->where('receiver_id', $authId);
        })
        ->orderBy('created_at', 'asc')
        ->get();

        // Mark messages as seen
        Message::where('sender_id', $userId)
            ->where('receiver_id', $authId)
            ->whereNull('seen_at')
            ->update(['seen_at' => now()]);

        return response()->json($messages);
    }

    // Send a message (text, media, reply)
    public function send(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'receiver_id' => 'required|exists:users,id',
            'message' => 'nullable|string',
            'type' => 'required|in:text,image,video,file',
            'file' => 'nullable|file|max:10240' // 10MB max
        ]);

        if ($validator->fails()) {
            return response()->json(['errors' => $validator->errors()], 422);
        }

        $filePath = null;
        if ($request->hasFile('file')) {
            $file = $request->file('file');
            $type = $request->type;
            $folder = 'chat/' . $type . 's';
            $filePath = $file->store($folder, 'public');
        }

        $message = Message::create([
            'sender_id' => Auth::id(),
            'receiver_id' => $request->receiver_id,
            'message' => $request->message,
            'type' => $request->type,
            'file_path' => $filePath
        ]);

        return response()->json($message);
    }

    // Get unread message count
    public function unread()
    {
        $count = Message::where('receiver_id', Auth::id())
            ->whereNull('seen_at')
            ->count();

        return response()->json(['count' => $count]);
    }

    // Mark a message as seen
    public function seen($messageId)
    {
        $message = Message::findOrFail($messageId);
        
        if ($message->receiver_id !== Auth::id()) {
            return response()->json(['error' => 'Unauthorized'], 403);
        }

        $message->update(['seen_at' => now()]);
        return response()->json(['success' => true]);
    }

    // List of conversations for the sidebar
    public function conversations()
    {
        $authId = Auth::id();
        $conversations = Message::selectRaw('IF(sender_id = ?, receiver_id, sender_id) as user_id', [$authId])
            ->where(function($q) use ($authId) {
                $q->where('sender_id', $authId)->orWhere('receiver_id', $authId);
            })
            ->groupBy('user_id')
            ->get()
            ->map(function($row) use ($authId) {
                $user = User::find($row->user_id);
                $lastMessage = Message::where(function($q) use ($authId, $row) {
                        $q->where('sender_id', $authId)->where('receiver_id', $row->user_id);
                    })->orWhere(function($q) use ($authId, $row) {
                        $q->where('sender_id', $row->user_id)->where('receiver_id', $authId);
                    })
                    ->orderByDesc('created_at')
                    ->first();
                $unread = Message::where('sender_id', $row->user_id)
                    ->where('receiver_id', $authId)
                    ->whereNull('seen_at')
                    ->count();
                return [
                    'id' => $user->id,
                    'name' => $user->name,
                    'email' => $user->email,
                    'last_message' => $lastMessage ? ($lastMessage->type === 'text' ? $lastMessage->message : ucfirst($lastMessage->type)) : '',
                    'last_message_time' => $lastMessage ? $lastMessage->created_at : null,
                    'unread' => $unread,
                    'online' => false
                ];
            });

        return response()->json($conversations);
    }

    // Search users for chat
    public function search(Request $request)
    {
        $query = $request->get('q');
        
        $users = User::where('id', '!=', Auth::id())
            ->where(function ($q) use ($query) {
                $q->where('name', 'like', "%{$query}%")
                    ->orWhere('email', 'like', "%{$query}%");
            })
            ->limit(10)
            ->get();

        return response()->json($users);
    }
} 