<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Models\DeliveryCompany;
use App\Services\DeliveryService;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class DeliveryWebhookController extends Controller
{
    public function __construct(protected DeliveryService $deliveryService)
    {
    }

    public function handle(Request $request, string $companyCode)
    {
        try {
            $company = DeliveryCompany::where('code', $companyCode)
                ->where('is_active', true)
                ->firstOrFail();

            // Verify webhook signature if provided
            if ($company->api_secret && !$this->verifySignature($request, $company->api_secret)) {
                Log::error('Invalid webhook signature', [
                    'company' => $company->name,
                    'headers' => $request->headers->all(),
                ]);
                return response()->json(['error' => 'Invalid signature'], 401);
            }

            $success = $this->deliveryService->handleWebhook($company, $request->all());

            return response()->json([
                'success' => $success,
                'message' => $success ? 'Webhook processed successfully' : 'Failed to process webhook',
            ]);
        } catch (\Exception $e) {
            Log::error('Webhook processing error', [
                'company' => $companyCode,
                'error' => $e->getMessage(),
                'data' => $request->all(),
            ]);

            return response()->json([
                'success' => false,
                'message' => 'Internal server error',
            ], 500);
        }
    }

    protected function verifySignature(Request $request, string $secret): bool
    {
        $signature = $request->header('X-Delivery-Signature');
        
        if (!$signature) {
            return false;
        }

        $payload = $request->getContent();
        $expectedSignature = hash_hmac('sha256', $payload, $secret);

        return hash_equals($expectedSignature, $signature);
    }
} 