<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Models\Order;
use App\Models\OrderItem;
use App\Models\ProductVariation;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Session;

class OrderController extends Controller
{
    public function index()
    {
        $orders = Order::with('items.variation.product')
            ->where('user_id', auth()->id())
            ->latest()
            ->get();

        return response()->json([
            'data' => $orders,
            'message' => 'Orders retrieved successfully'
        ]);
    }

    public function show(Order $order)
    {
        if ($order->user_id !== auth()->id()) {
            return response()->json([
                'message' => 'Unauthorized'
            ], 403);
        }

        $order->load('items.variation.product');

        return response()->json([
            'data' => $order,
            'message' => 'Order retrieved successfully'
        ]);
    }

    public function store(Request $request)
    {
        $request->validate([
            'shipping_name' => 'required|string',
            'shipping_email' => 'required|email',
            'shipping_phone' => 'required|string',
            'shipping_address' => 'required|string',
            'payment_method' => 'required|string|in:cash_on_delivery,card',
            'notes' => 'nullable|string'
        ]);

        $cart = Session::get('cart', []);
        if (empty($cart)) {
            return response()->json([
                'message' => 'Cart is empty'
            ], 400);
        }

        // Calculate totals
        $subtotal = 0;
        $items = [];

        foreach ($cart as $item) {
            $variation = ProductVariation::with('product')->find($item['variation_id']);
            
            if (!$variation || $variation->stock < $item['quantity']) {
                return response()->json([
                    'message' => 'Some items are no longer available'
                ], 400);
            }

            $price = $variation->product->current_price;
            $total = $price * $item['quantity'];

            $items[] = [
                'variation' => $variation,
                'quantity' => $item['quantity'],
                'price' => $price,
                'total' => $total
            ];

            $subtotal += $total;
        }

        // Create order
        $order = Order::create([
            'user_id' => auth()->id() ?? null,
            'order_number' => Order::generateOrderNumber(),
            'status' => 'pending',
            'subtotal' => $subtotal,
            'total' => $subtotal,
            'payment_method' => $request->payment_method,
            'payment_status' => 'pending',
            'shipping_name' => $request->shipping_name,
            'shipping_email' => $request->shipping_email,
            'shipping_phone' => $request->shipping_phone,
            'shipping_address' => $request->shipping_address,
            'notes' => $request->notes
        ]);

        // Create order items
        foreach ($items as $item) {
            OrderItem::create([
                'order_id' => $order->id,
                'product_id' => $item['variation']->product_id,
                'variation_id' => $item['variation']->id,
                'quantity' => $item['quantity'],
                'price' => $item['price'],
                'total' => $item['total']
            ]);

            // Update stock
            $item['variation']->decrement('stock', $item['quantity']);
        }

        // Clear cart
        Session::forget('cart');

        return response()->json([
            'data' => $order->load('items.variation.product'),
            'message' => 'Order created successfully'
        ]);
    }
} 