<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use App\Enums\OrderStatus;
use App\Traits\LogsActivity;

class Order extends Model
{
    use HasFactory, LogsActivity;

    protected $fillable = [
        'user_id',
        'order_number',
        'status',
        'payment_method',
        'payment_status',
        'shipping_name',
        'shipping_email',
        'shipping_phone',
        'shipping_address',
        'shipping_city',
        'shipping_state',
        'shipping_country',
        'shipping_postal_code',
        'subtotal',
        'discount',
        'total',
        'shipping_cost',
        'shipping_method',
        'delivery_company',
        'tracking_number',
        'tracking_url',
        'shipped_at',
        'delivered_at',
        'delivery_notes',
        'notes'
    ];

    protected $casts = [
        'shipped_at' => 'datetime',
        'delivered_at' => 'datetime',
        'status' => OrderStatus::class
    ];

    public function user()
    {
        return $this->belongsTo(User::class);
    }

    public function customer()
    {
        return $this->belongsTo(User::class, 'user_id');
    }

    public function items()
    {
        return $this->hasMany(OrderItem::class);
    }

    public function deliveryCompany()
    {
        return $this->belongsTo(DeliveryCompany::class, 'delivery_company', 'code');
    }

    public function getTotalAmountAttribute()
    {
        return $this->total + $this->shipping_cost;
    }

    public function getTrackingUrlAttribute()
    {
        if (!$this->tracking_number || !$this->delivery_company) {
            return null;
        }

        return $this->deliveryCompany?->getTrackingUrl($this->tracking_number);
    }

    public function getShippingAddressFormattedAttribute()
    {
        $parts = array_filter([
            $this->shipping_address,
            $this->shipping_city,
            $this->shipping_state,
            $this->shipping_postal_code,
            $this->shipping_country
        ]);

        return implode(', ', $parts);
    }

    public static function generateOrderNumber()
    {
        $prefix = 'ORD';
        $date = now()->format('Ymd');
        $random = str_pad(rand(0, 9999), 4, '0', STR_PAD_LEFT);
        return $prefix . $date . $random;
    }

    public function canBeShipped()
    {
        return $this->status === OrderStatus::PROCESSING && 
               $this->payment_status === 'paid' &&
               !$this->shipped_at;
    }

    public function canBeDelivered()
    {
        return $this->status === OrderStatus::SHIPPED && 
               $this->shipped_at &&
               !$this->delivered_at;
    }
} 