<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use App\Traits\LogsActivity;

class Product extends Model
{
    use HasFactory, LogsActivity;

    protected $fillable = [
        'category_id',
        'name_en',
        'name_ar',
        'slug',
        'description_en',
        'description_ar',
        'is_featured',
        'is_active'
    ];

    protected $casts = [
        'is_featured' => 'boolean',
        'is_active' => 'boolean'
    ];

    public function category()
    {
        return $this->belongsTo(Category::class);
    }

    public function variations()
    {
        return $this->hasMany(Variation::class);
    }

    public function variationTypes()
    {
        return $this->belongsToMany(VariationType::class, 'product_variation_types');
    }

    public function images()
    {
        return $this->hasMany(ProductImage::class);
    }

    public function orderItems()
    {
        return $this->hasMany(OrderItem::class);
    }

    public function getNameAttribute()
    {
        return app()->getLocale() === 'ar' ? $this->name_ar : $this->name_en;
    }

    public function getDescriptionAttribute()
    {
        return app()->getLocale() === 'ar' ? $this->description_ar : $this->description_en;
    }

    public function getIsOnDiscountAttribute()
    {
        if (!$this->discount_price || !$this->discount_start || !$this->discount_end) {
            return false;
        }

        $now = now();
        return $now->between($this->discount_start, $this->discount_end);
    }

    public function getCurrentPriceAttribute()
    {
        return $this->is_on_discount ? $this->discount_price : $this->price;
    }

    public function getTimeLeftForDiscountAttribute()
    {
        if (!$this->is_on_discount) {
            return null;
        }

        return $this->discount_end->diffInSeconds(now());
    }

    public function getImageUrlAttribute()
    {
        $primaryImage = $this->images()->where('is_primary', true)->first();
        if ($primaryImage) {
            return asset('storage/' . $primaryImage->image_path);
        }
        
        // If no primary image, get the first image
        $firstImage = $this->images()->first();
        if ($firstImage) {
            return asset('storage/' . $firstImage->image_path);
        }
        
        // If no images at all, return a default image
        return asset('images/default-product.png');
    }
} 