<?php

namespace App\Services;

use App\Models\Order;
use App\Models\DeliveryCompany;
use App\Notifications\OrderStatusUpdated;
use Illuminate\Support\Facades\Http;
use Illuminate\Support\Facades\Log;

class DeliveryService
{
    public function createShipment(Order $order, DeliveryCompany $company): bool
    {
        try {
            $response = Http::withHeaders([
                'Authorization' => "Bearer {$company->api_key}",
                'Content-Type' => 'application/json',
            ])->post($company->webhook_url . '/shipments', [
                'order_id' => $order->id,
                'tracking_number' => $order->tracking_number,
                'shipping_address' => [
                    'name' => $order->shipping_name,
                    'address' => $order->shipping_address,
                    'city' => $order->shipping_city,
                    'state' => $order->shipping_state,
                    'country' => $order->shipping_country,
                    'postal_code' => $order->shipping_postal_code,
                    'phone' => $order->shipping_phone,
                    'email' => $order->shipping_email,
                ],
                'items' => $order->items->map(fn($item) => [
                    'name' => $item->product->name,
                    'quantity' => $item->quantity,
                    'weight' => $item->product->weight ?? 1,
                ])->toArray(),
            ]);

            if ($response->successful()) {
                $data = $response->json();
                $order->update([
                    'tracking_number' => $data['tracking_number'],
                    'tracking_url' => $data['tracking_url'],
                    'shipped_at' => now(),
                ]);

                $order->user->notify(new OrderStatusUpdated($order));
                return true;
            }

            Log::error('Failed to create shipment', [
                'order_id' => $order->id,
                'company' => $company->name,
                'response' => $response->json(),
            ]);

            return false;
        } catch (\Exception $e) {
            Log::error('Exception while creating shipment', [
                'order_id' => $order->id,
                'company' => $company->name,
                'error' => $e->getMessage(),
            ]);

            return false;
        }
    }

    public function updateOrderStatus(Order $order, string $status, ?string $trackingNumber = null): bool
    {
        try {
            $oldStatus = $order->status;
            
            if (!$order->status->canTransitionTo(OrderStatus::from($status))) {
                Log::error('Invalid status transition', [
                    'order_id' => $order->id,
                    'from' => $oldStatus,
                    'to' => $status,
                ]);
                return false;
            }

            $updates = ['status' => $status];

            if ($trackingNumber) {
                $updates['tracking_number'] = $trackingNumber;
                $updates['tracking_url'] = $order->deliveryCompany?->getTrackingUrl($trackingNumber);
            }

            if ($status === 'shipped') {
                $updates['shipped_at'] = now();
            } elseif ($status === 'delivered') {
                $updates['delivered_at'] = now();
            }

            $order->update($updates);
            $order->user->notify(new OrderStatusUpdated($order));

            // Log the status change
            activity()
                ->performedOn($order)
                ->withProperties([
                    'old_status' => $oldStatus,
                    'new_status' => $status,
                    'tracking_number' => $trackingNumber,
                ])
                ->log('Order status updated');

            return true;
        } catch (\Exception $e) {
            Log::error('Exception while updating order status', [
                'order_id' => $order->id,
                'status' => $status,
                'error' => $e->getMessage(),
            ]);

            return false;
        }
    }

    public function handleWebhook(DeliveryCompany $company, array $data): bool
    {
        try {
            $order = Order::where('tracking_number', $data['tracking_number'])->first();
            
            if (!$order) {
                Log::error('Order not found for webhook', [
                    'company' => $company->name,
                    'tracking_number' => $data['tracking_number'],
                ]);
                return false;
            }

            return $this->updateOrderStatus($order, $data['status']);
        } catch (\Exception $e) {
            Log::error('Exception while handling webhook', [
                'company' => $company->name,
                'data' => $data,
                'error' => $e->getMessage(),
            ]);

            return false;
        }
    }
} 