<?php

namespace App\Services;

use App\Models\Order;
use App\Models\ProductVariation;
use Illuminate\Support\Collection;
use Illuminate\Support\Facades\Log;

class InventoryService
{
    public function checkInventory(Order $order): array
    {
        $issues = [];
        $items = $order->items;

        foreach ($items as $item) {
            $variation = ProductVariation::find($item->variation_id);
            
            if (!$variation) {
                $issues[] = [
                    'type' => 'missing_variation',
                    'message' => "Product variation not found for item #{$item->id}",
                    'item_id' => $item->id,
                ];
                continue;
            }

            if ($variation->stock < $item->quantity) {
                $issues[] = [
                    'type' => 'insufficient_stock',
                    'message' => "Insufficient stock for {$variation->product->name} ({$variation->size_en} - {$variation->color_en})",
                    'item_id' => $item->id,
                    'required' => $item->quantity,
                    'available' => $variation->stock,
                ];
            }
        }

        return $issues;
    }

    public function reserveInventory(Order $order): bool
    {
        try {
            $items = $order->items;
            $reservations = new Collection();

            foreach ($items as $item) {
                $variation = ProductVariation::find($item->variation_id);
                
                if (!$variation || $variation->stock < $item->quantity) {
                    // Release any already reserved items
                    $this->releaseReservations($reservations);
                    return false;
                }

                $variation->decrement('stock', $item->quantity);
                $reservations->push([
                    'variation_id' => $variation->id,
                    'quantity' => $item->quantity,
                ]);
            }

            return true;
        } catch (\Exception $e) {
            Log::error('Failed to reserve inventory', [
                'order_id' => $order->id,
                'error' => $e->getMessage(),
            ]);

            // Release any already reserved items
            $this->releaseReservations($reservations);
            return false;
        }
    }

    protected function releaseReservations(Collection $reservations): void
    {
        foreach ($reservations as $reservation) {
            ProductVariation::where('id', $reservation['variation_id'])
                ->increment('stock', $reservation['quantity']);
        }
    }
} 