<?php

namespace App\Services;

use App\Models\Order;
use App\Notifications\OrderAlert;
use App\Enums\OrderStatus;
use Carbon\Carbon;
use Illuminate\Support\Facades\Log;

class OrderMonitoringService
{
    protected const PENDING_THRESHOLD_DAYS = 3;
    protected const PROCESSING_THRESHOLD_DAYS = 2;
    protected const SHIPPED_THRESHOLD_DAYS = 5;

    public function checkPendingOrders(): void
    {
        $threshold = Carbon::now()->subDays(self::PENDING_THRESHOLD_DAYS);
        
        Order::where('status', OrderStatus::PENDING)
            ->where('created_at', '<', $threshold)
            ->whereDoesntHave('notifications', function ($query) {
                $query->where('type', OrderAlert::class)
                    ->where('data->alert_type', 'pending_threshold');
            })
            ->each(function ($order) {
                $this->sendAlert($order, 'pending_threshold', 'Order pending for too long');
            });
    }

    public function checkProcessingOrders(): void
    {
        $threshold = Carbon::now()->subDays(self::PROCESSING_THRESHOLD_DAYS);
        
        Order::where('status', OrderStatus::PROCESSING)
            ->where('created_at', '<', $threshold)
            ->whereDoesntHave('notifications', function ($query) {
                $query->where('type', OrderAlert::class)
                    ->where('data->alert_type', 'processing_threshold');
            })
            ->each(function ($order) {
                $this->sendAlert($order, 'processing_threshold', 'Order processing for too long');
            });
    }

    public function checkShippedOrders(): void
    {
        $threshold = Carbon::now()->subDays(self::SHIPPED_THRESHOLD_DAYS);
        
        Order::where('status', OrderStatus::SHIPPED)
            ->where('shipped_at', '<', $threshold)
            ->whereDoesntHave('notifications', function ($query) {
                $query->where('type', OrderAlert::class)
                    ->where('data->alert_type', 'shipped_threshold');
            })
            ->each(function ($order) {
                $this->sendAlert($order, 'shipped_threshold', 'Order shipped but not delivered');
            });
    }

    public function checkFailedDeliveries(): void
    {
        Order::where('status', OrderStatus::SHIPPED)
            ->whereHas('deliveryCompany', function ($query) {
                $query->where('code', 'aramex')
                    ->orWhere('code', 'dhl');
            })
            ->whereDoesntHave('notifications', function ($query) {
                $query->where('type', OrderAlert::class)
                    ->where('data->alert_type', 'failed_delivery');
            })
            ->each(function ($order) {
                // Check delivery company API for failed delivery status
                if ($this->checkDeliveryStatus($order)) {
                    $this->sendAlert($order, 'failed_delivery', 'Delivery attempt failed');
                }
            });
    }

    protected function sendAlert(Order $order, string $type, string $message): void
    {
        try {
            $order->user->notify(new OrderAlert($order, $type, $message));
            
            Log::info('Order alert sent', [
                'order_id' => $order->id,
                'alert_type' => $type,
                'message' => $message,
            ]);
        } catch (\Exception $e) {
            Log::error('Failed to send order alert', [
                'order_id' => $order->id,
                'alert_type' => $type,
                'error' => $e->getMessage(),
            ]);
        }
    }

    protected function checkDeliveryStatus(Order $order): bool
    {
        // Implement delivery company API check
        // This is a placeholder - you'll need to implement the actual API calls
        return false;
    }
} 