<?php

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Route;
use App\Http\Controllers\Api\ProductController;
use App\Http\Controllers\Api\CategoryController;
use App\Http\Controllers\Api\CartController;
use App\Http\Controllers\Api\OrderController;
use App\Http\Controllers\Api\AuthController;
use App\Http\Controllers\Api\LanguageController;
use App\Http\Controllers\Admin\ProductController as AdminProductController;
use App\Http\Controllers\Admin\CategoryController as AdminCategoryController;
use App\Http\Controllers\Admin\OrderController as AdminOrderController;
use App\Http\Controllers\Admin\ChatController as AdminChatController;
use App\Http\Controllers\Api\ChatController;
use App\Http\Controllers\Api\DeliveryWebhookController;

// Public routes
Route::get('/categories', [CategoryController::class, 'index']);
Route::get('/categories/{category}', [CategoryController::class, 'show']);
Route::get('/products', [ProductController::class, 'index']);
Route::get('/products/{product}', [ProductController::class, 'show']);
Route::get('/products/featured', [ProductController::class, 'featured']);
Route::post('/language/switch', [LanguageController::class, 'switch']);

// Cart routes (no auth required)
Route::get('/cart', [CartController::class, 'index']);
Route::post('/cart/add', [CartController::class, 'add']);
Route::put('/cart/update/{item}', [CartController::class, 'update']);
Route::delete('/cart/remove/{item}', [CartController::class, 'remove']);

// Order creation (no auth required)
Route::post('/orders', [OrderController::class, 'store']);

// Auth routes
Route::post('/register', [AuthController::class, 'register']);
Route::post('/login', [AuthController::class, 'login']);

// Protected routes
Route::middleware(['auth:sanctum'])->group(function () {
    // User routes
    Route::get('/user', function (Request $request) {
        return $request->user();
    });
    Route::post('/logout', [AuthController::class, 'logout']);

    // Chat routes
    Route::prefix('chat')->group(function () {
        Route::get('/conversations', [ChatController::class, 'conversations']);
        Route::get('/messages/{user}', [ChatController::class, 'thread']);
        Route::post('/send', [ChatController::class, 'send']);
        Route::get('/search', [ChatController::class, 'search']);
        Route::post('/messages/{message}/reactions', [ChatController::class, 'toggleReaction']);
        Route::delete('/messages/{message}', [ChatController::class, 'deleteMessage']);
        Route::put('/messages/{message}', [ChatController::class, 'editMessage']);
        Route::get('/unread', [ChatController::class, 'unread']);
        Route::post('/messages/{message}/seen', [ChatController::class, 'seen']);
    });

    // Order history routes (auth required)
    Route::get('/orders', [OrderController::class, 'index']);
    Route::get('/orders/{order}', [OrderController::class, 'show']);

    // Admin routes
    Route::middleware('admin')->prefix('admin')->group(function () {
        // Product management
        Route::apiResource('products', AdminProductController::class);

        // Category management
        Route::apiResource('categories', AdminCategoryController::class);

        // Order management
        Route::get('orders', [AdminOrderController::class, 'index']);
        Route::get('orders/{order}', [AdminOrderController::class, 'show']);
        Route::put('orders/{order}/status', [AdminOrderController::class, 'updateStatus']);
        Route::put('orders/{order}/payment-status', [AdminOrderController::class, 'updatePaymentStatus']);

        // Chat routes
        Route::prefix('chat')->group(function () {
            Route::get('/conversations', [AdminChatController::class, 'getConversations']);
            Route::get('/messages/{user}', [AdminChatController::class, 'getMessages']);
            Route::post('/send', [AdminChatController::class, 'sendMessage']);
            Route::get('/search', [AdminChatController::class, 'search']);
            Route::post('/messages/{message}/reactions', [AdminChatController::class, 'addReaction']);
            Route::delete('/messages/{message}', [AdminChatController::class, 'deleteMessage']);
            Route::put('/messages/{message}', [AdminChatController::class, 'editMessage']);
            Route::get('/unread', [AdminChatController::class, 'getUnreadCount']);
            Route::post('/messages/{message}/seen', [AdminChatController::class, 'markAsSeen']);
        });
    });
});

// Delivery webhooks
Route::post('webhooks/delivery/{company}', [DeliveryWebhookController::class, 'handle'])
    ->name('webhooks.delivery'); 